@@ -1,8 +1,10 @@
 /************************************************************************
 **
+**  Copyright (C) 2015-2025 Kevin B. Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2015-2022 Doug Massay
 **  Copyright (C) 2012-2015 John Schember <john@nachtimwald.com>
-**  Copyright (C) 2012 Dave Heiland
-**  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2012      Dave Heiland
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -25,8 +27,8 @@
 #ifndef SIGIL_H
 #define SIGIL_H
 
-#include <QtCore/QSharedPointer>
-#include <QtWidgets/QMainWindow>
+#include <QSharedPointer>
+#include <QMainWindow>
 
 #include "ui_main.h"
 #include "BookManipulation/Book.h"
@@ -38,7 +40,7 @@
 #include "Dialogs/ViewImage.h"
 #include "MainUI/FindReplace.h"
 #include "MainUI/TOCModel.h"
-#include "Misc/CSSInfo.h"
+#include "Parsers/CSSInfo.h"
 #include "Misc/PasteTarget.h"
 #include "Misc/SettingsStore.h"
 #include "Misc/ValidationResult.h"
@@ -46,8 +48,10 @@
 #include "MiscEditors/IndexEditorModel.h"
 #include "MiscEditors/SearchEditorModel.h"
 #include "Tabs/ContentTab.h"
+#include "ViewEditors/ElementIndex.h"
+#include "Dialogs/SearchEditor.h"
 
-const int MAX_RECENT_FILES = 5;
+const int MAX_RECENT_FILES = 9;
 const int STATUSBAR_MSG_DISPLAY_TIME = 7000;
 
 class QComboBox;
@@ -55,6 +59,7 @@ class QLabel;
 class QSignalMapper;
 class QSlider;
 class QTimer;
+class QActionGroup;
 class FindReplace;
 class TabManager;
 class BookBrowser;
@@ -67,6 +72,7 @@ class ClipsWindow;
 class SelectCharacter;
 class ViewImage;
 class FlowTab;
+class QWidget;
 
 
 /**
@@ -99,9 +105,20 @@ public:
      * @param parent The window's parent object.
      * @param flags The flags used to modify window behavior.
      */
-    MainWindow(const QString &openfilepath = QString(), bool is_internal = false, QWidget *parent = 0, Qt::WindowFlags flags = 0);
+    MainWindow(const QString &openfilepath = QString(), 
+               const QString version = QString(),
+               bool is_internal = false, 
+               QWidget *parent = 0, 
+               Qt::WindowFlags flags = Qt::WindowFlags());
+
     ~MainWindow();
 
+    // routines to help Find and Replace exchange info with the SearchEditor
+    QList<SearchEditorModel::searchEntry*> SearchEditorGetCurrentEntries();
+    void SearchEditorRecordEntryAsCompleted(SearchEditorModel::searchEntry* entry);
+
+    void maybe_fixup_dockwidget_geometry(QDockWidget * widget);
+
     /**
      * The book currently being edited.
      *
@@ -132,7 +149,17 @@ public:
      *
      * @return List of valid selected HTML resources
      */
+    QList <Resource *> GetBookBrowserSelectedResources();
     QList <Resource *> GetValidSelectedHTMLResources();
+    QList <Resource *> GetValidSelectedCSSResources();
+    QList <Resource *> GetValidSelectedSVGResources();
+    QList <Resource *> GetValidSelectedJSResources();
+    QList <Resource *> GetValidSelectedMiscXMLResources();
+
+    QList <Resource *> GetTabbedHTMLResources();
+    QList <Resource *> GetTabbedCSSResources();
+
+    bool RenameClassInHtml(const QString& oldname, const QString& newname);
 
     /**
      * Returns a list of all HTML resources in book browser order
@@ -140,7 +167,13 @@ public:
      * @return List of all HTML resources in book browser order
      */
     QList <Resource *> GetAllHTMLResources();
+    QList <Resource *> GetAllCSSResources();
 
+    QList <Resource *> GetOPFResource();
+    QList <Resource *> GetNCXResource();
+
+    /* returns the OPF metadata tag and all its children as xml */
+    QString GetOPFMetadataXML();
 
     /**
      * Select resources in the Book Browser
@@ -148,33 +181,15 @@ public:
      */
     void SelectResources(QList<Resource *> resources);
 
-    /**
-     * Describes the type of the View mode
-     * currently used in FlowTab.
-     */
-    enum ViewState {
-        ViewState_Unknown = 0,     /**< Default non view that we don't know or care what it is */
-        ViewState_BookView = 10,   /**< The WYSIWYG view. */
-        ViewState_CodeView = 30    /**< The XHTML code editing view. */
-    };
-
     /**
      * The location of the last bookmark.
      */
     struct LocationBookmark {
-        QString filename;
-        MainWindow::ViewState view_state;
+        QString bookpath;
         QString bv_caret_location_update;
         int cv_cursor_position;
     };
 
-    /**
-     * Returns the current view state.
-     *
-     * @return The current view state.
-     */
-    MainWindow::ViewState GetViewState();
-
     void CloseAllTabs();
 
     void SaveTabData();
@@ -209,12 +224,22 @@ public:
 
     QString GetCurrentFilePath();
 
+    void DebugCurrentWidgetSizes();
+
+    QString GetMathJaxFolder() { return m_mathjaxfolder; };
+
+    bool UsingAutomate() { return m_UsingAutomate; }
+
+    QString AutomatePluginParameter() { return m_AutomatePluginParameter; }
 
 public slots:
-    void AnyCodeView();
+
+    bool Automate(const QStringList &commands);
 
     void OpenUrl(const QUrl &url);
 
+    void ScrollCVToFragment(const QString &fragment);
+
     /**
      * Opens the specified resource in the specified view state.
      */
@@ -222,7 +247,6 @@ public slots:
                       int line_to_scroll_to = -1,
                       int position_to_scroll_to = -1,
                       const QString &caret_location_to_scroll_to = QString(),
-                      MainWindow::ViewState view_state = MainWindow::ViewState_Unknown,
                       const QUrl &fragment = QUrl(),
                       bool precede_current_tab = false);
 
@@ -230,26 +254,57 @@ public slots:
                                         int line_to_scroll_to = -1,
                                         int position_to_scroll_to = -1,
                                         const QString &caret_location_to_scroll_to = QString(),
-                                        MainWindow::ViewState view_state = MainWindow::ViewState_Unknown,
                                         const QUrl &fragment = QUrl(),
                                         bool precede_current_tab = false);
 
-    void UpdateManifestProperties();
+    bool UpdateManifestProperties();
 
-    void GenerateNCXFromNav();
+    bool UpdateManifestMediaTypes();
+
+    bool GenerateNCXGuideFromNav();
+
+    bool RemoveNCXGuideFromEpub3();
 
     void CreateIndex();
 
     void runPlugin(QAction *action);
 
-    void ResourcesAddedOrDeleted();
+    void ResourcesAddedOrDeletedOrMoved();
 
+    void launchExternalXEditor();
+
+    bool RepoCommit();
+    void RepoCheckout(QString bookid="", QString destpath="", QString filename="", bool loadnow=true);
+    void RepoDiff(QString bookid="");
+    void RepoManage();
+    void RepoEditTagDescription();
+    void RepoShowLog();
+
+    void runAutomate(QAction *action);
+    void RunAutomate(const QString &automatefile);
+    void UpdateAutomationMenu();
+    void ManageAutomateListsDialog();
+    
+    bool StandardizeEpub();
+    bool UseStandardFileExtensions();
+    bool RebaseManifestIDs();
+
+    void CreateEpubLayout();
+
+    void FocusOnCodeView();
+    void FocusOnBookBrowser();
+    void FocusOnPreview();
+    void FocusOnTOC();
+    void FocusOnClips();
 
 signals:
     void SettingsChanged();
 
 protected:
     void showEvent(QShowEvent *event);
+
+    void changeEvent(QEvent * event);
+
     /**
      * Workaround for Qt 4.8 bug, which does not save/restore window state
      * correctly if maximized at the time of calling saveGeometry().
@@ -267,12 +322,15 @@ protected:
 
 private slots:
 
-    void AddCover();
+    bool AddCover();
 
     /**
      * Implements New action functionality.
      */
-    void New();
+    void New(const QString version = QString());
+    void NewDefault();
+    void NewEpub2();
+    void NewEpub3();
 
     /**
      * Implements Open action functionality.
@@ -337,8 +395,8 @@ private slots:
 
     bool DeleteCSSStyles(const QString &filename, QList<CSSInfo::CSSSelector *> css_selectors);
 
-    void DeleteUnusedMedia();
-    void DeleteUnusedStyles();
+    bool DeleteUnusedMedia(bool in_automate = false);
+    bool DeleteUnusedStyles(bool in_automate = false);
 
     void InsertFileDialog();
 
@@ -346,6 +404,10 @@ private slots:
 
     void InsertId();
 
+    void InsertClip();
+
+    void InsertRole();
+
     void InsertHyperlink();
 
     void MarkForIndex();
@@ -356,13 +418,17 @@ private slots:
      */
     void ApplicationFocusChanged(QWidget *old, QWidget *now);
 
+    void ApplicationPaletteChanged();
+
     /** 
      *  Quick Launch Plugins via icon button
      */
     void QuickLaunchPlugin(int i);
 
+    void QuickLaunchAutomate(int i);
+
     /**
-     * Some controls (CodeView, BookView and combo boxes in F&R) inherit PasteTarget
+     * Some controls (CodeView and combo boxes in F&R) inherit PasteTarget
      * to allow various modeless/popup dialogs like Clipboard History, Clip Editor and
      * Insert Special Characters to insert text into the focused "PasteTarget" control.
      * These two slots will delegate the relevant signal to the current target if any.
@@ -371,31 +437,6 @@ private slots:
     void PasteClipEntriesIntoCurrentTarget(const QList<ClipEditorModel::clipEntry *> &clips);
     void PasteClipEntriesIntoPreviousTarget(const QList<ClipEditorModel::clipEntry *> &clips);
     void PasteClipIntoCurrentTarget(int clip_number);
-    void PasteClip1IntoCurrentTarget();
-    void PasteClip2IntoCurrentTarget();
-    void PasteClip3IntoCurrentTarget();
-    void PasteClip4IntoCurrentTarget();
-    void PasteClip5IntoCurrentTarget();
-    void PasteClip6IntoCurrentTarget();
-    void PasteClip7IntoCurrentTarget();
-    void PasteClip8IntoCurrentTarget();
-    void PasteClip9IntoCurrentTarget();
-    void PasteClip10IntoCurrentTarget();
-    void PasteClip11IntoCurrentTarget();
-    void PasteClip12IntoCurrentTarget();
-    void PasteClip13IntoCurrentTarget();
-    void PasteClip14IntoCurrentTarget();
-    void PasteClip15IntoCurrentTarget();
-    void PasteClip16IntoCurrentTarget();
-    void PasteClip17IntoCurrentTarget();
-    void PasteClip18IntoCurrentTarget();
-    void PasteClip19IntoCurrentTarget();
-    void PasteClip20IntoCurrentTarget();
-
-    /**
-     * Implements the set BookView functionality.
-     */
-    void BookView();
 
     /**
      * Implements the set CodeView functionality.
@@ -414,6 +455,11 @@ private slots:
 
     void ClipEditorDialog(ClipEditorModel::clipEntry *clip_entry = NULL);
 
+    /**
+     * Allow Keyboard Focus Navigation to main QDockWidgets
+     */
+    void FocusOn(QWidget* dw=nullptr);
+  
     /**
      * Implements User Guide action functionality.
      */
@@ -448,9 +494,11 @@ private slots:
     /**
      * Implements Validate Epub action functionality.
      */
-    void WellFormedCheckEpub();
+    bool WellFormedCheckEpub();
 
-    void ValidateStylesheetsWithW3C();
+    bool ValidateStylesheetsWithW3C();
+
+    bool ReformatAllStylesheets(bool multiple_line_format);
 
     bool CharLessThan(const QChar &s1, const QChar &s2);
 
@@ -468,7 +516,7 @@ private slots:
      * Updates the toolbars/menus based on current state
      * and updates the tab state if requested
      */
-    void UpdateViewState(bool set_tab_state = true);
+    void UpdateMWState(bool set_tab_state = true);
 
     /**
      * Updates the toolbars based on current tab state and changes.
@@ -485,11 +533,6 @@ private slots:
      */
     void UpdateUIWhenTabsSwitch();
 
-    /**
-     * Set initial state for actions in Book View
-     */
-    void SetStateActionsBookView();
-
     /**
      * Set initial state for actions in Code View
      */
@@ -512,7 +555,8 @@ private slots:
     void SetStateActionsStaticView();
 
     void UpdatePreviewRequest();
-    void UpdatePreviewCSSRequest();
+    void UpdatePreviewTabRequest();
+    void ScrollPreview();
     void UpdatePreview();
     void InspectHTML();
 
@@ -562,7 +606,7 @@ private slots:
      * @param originating_resource  The original resource from which the content
      *                              was extracted to create the "old" tab/resource.
      * @see FlowTab::SplitSection, FlowTab::OldTabRequest,
-     *      BookViewEditor::SplitSection, Book::CreateSectionBreakOriginalResource
+     *      Book::CreateSectionBreakOriginalResource
      */
     void CreateSectionBreakOldTab(QString content, HTMLResource *originating_resource);
 
@@ -577,15 +621,15 @@ private slots:
     /**
      * Creates new section/XHTML documents.
      */
-    void SplitOnSGFSectionMarkers();
+    bool SplitOnSGFSectionMarkers();
 
     void SetAutoSpellCheck(bool new_state);
 
     /**
      * Reformats all the book's html resources using Book.cpp's ReformatAllHTML() function.
      */
-    void MendPrettifyHTML();
-    void MendHTML();
+    bool MendPrettifyHTML();
+    bool MendHTML();
 
     void ClearIgnoredWords();
 
@@ -595,6 +639,8 @@ private slots:
 
     void LinkStylesheetsToResources(QList <Resource *> resources);
 
+    void LinkJavascriptsToResources(QList <Resource *> resources);
+
     void ResourceUpdatedFromDisk(Resource *resource);
 
     void UpdateWord(QString old_word, QString new_word);
@@ -610,20 +656,22 @@ private slots:
      */
     QStringList GetStylesheetsAlreadyLinked(Resource *resource);
 
+    QList<std::pair<QString, bool>> GetJavascriptsMap(QList<Resource *> resources);
+    QStringList GetJavascriptsAlreadyLinked(Resource *resource);
+
+
     void RemoveResources(QList<Resource *> resources = QList<Resource *>());
 
-    void GenerateToc();
+    bool GenerateTOC(bool skip_selector = false);
+    bool CreateHTMLTOC();
     void EditTOCDialog();
-    void CreateHTMLTOC();
 
-    void ChangeCasing(int casing_mode);
+    void ChangeCasing(QAction* act);
 
     void MarkSelection();
     void ClearMarkedText(ContentTab *old_tab = NULL);
 
-    void ToggleViewState();
-
-    void ApplyHeadingStyleToTab(const QString &heading_type);
+    void ApplyHeadingStyleToTab(QAction* act);
     void SetPreserveHeadingAttributes(bool new_state);
 
     void GoBackFromLinkOrStyle();
@@ -645,7 +693,8 @@ private slots:
     void DeleteReportsStyles(QList<BookReports::StyleData *> reports_styles_to_delete);
 
     void DeleteFilenames(QStringList files_to_delete);
-    void OpenFile(QString filename, int line = -1);
+
+    void OpenFile(QString file_bookpath, int line = -1, int position = -1);
 
     void UpdateClipsUI();
 
@@ -657,7 +706,8 @@ private slots:
 
 private:
     void updateToolTipsOnPluginIcons();
-    void UpdateClipButton(int clip_number, QAction *ui_action);
+    void updateToolTipsOnAutomateIcons();
+    void UpdateClipButton(QAction *ui_action);
     void InsertFiles(const QStringList &selected_images);
     void InsertFilesFromDisk();
 
@@ -676,8 +726,6 @@ private:
      */
     void WriteSettings();
 
-    void SetDefaultViewState();
-
     /**
      * Gets called on possible saves and asks the user
      * does he want to save.
@@ -689,6 +737,8 @@ private:
      */
     bool MaybeSaveDialogSaysProceed();
 
+    bool ProceedToOverwrite(const QString &msg, const QString &filename );
+
     /**
      * Makes the provided book the current one.
      *
@@ -700,7 +750,7 @@ private:
      * Creates a new, empty book and replaces
      * the current one with it.
      */
-    void CreateNewBook();
+    void CreateNewBook(const QString version=QString(), const QStringList &book_paths=QStringList());
 
     /**
      * Saves the current book to the file specified.
@@ -759,7 +809,7 @@ private:
      *
      * @param fullfilepath The path to the currently edited file.
      */
-    void UpdateUiWithCurrentFile(const QString &fullfilepath);
+    void UpdateUiWithCurrentFile(const QString &fullfilepath, bool just_name = false);
 
     /**
      * Selects the appropriate entry in the heading combo box
@@ -786,6 +836,11 @@ private:
      */
     void PlatformSpecificTweaks();
 
+    /**
+     * Tweak default UI fonts based on the OS platform.
+     */
+    void SetupUiFont();
+
     /**
      * Extends the UI with extra widgets and tweaks.
      * Qt Designer is not able to create all the widgets
@@ -809,7 +864,7 @@ private:
      *
      * @param openfilepath The path to the file to load. Can be empty.
      */
-    void LoadInitialFile(const QString &openfilepath, bool is_internal = false);
+    void LoadInitialFile(const QString &openfilepath, const QString version=QString(), bool is_internal = false);
 
     /**
      * Connects all the required signals to their slots.
@@ -830,15 +885,11 @@ private:
      */
     void BreakTabConnections(ContentTab *tab);
 
-    /**
-     * Sets the view state of the current tab to view_state
-     *
-     * @param view_state - The view state to set.
-     */
-    void SetViewState(MainWindow::ViewState view_state);
-
     void SetupPreviewTimer();
 
+    void FixDuplicateFilenames();
+    void MoveContentFilesToStdFolders();
+
     ///////////////////////////////
     // PRIVATE MEMBER VARIABLES
     ///////////////////////////////
@@ -852,6 +903,8 @@ private:
     QStringList m_LastOpenFileWarnings;
     bool m_IsInitialLoad;
 
+    QByteArray m_LastState;
+    
     /**
      * The path to the current file loaded.
      */
@@ -952,17 +1005,10 @@ private:
      */
     const QMap<QString, QString> c_LoadFilters;
 
-    /**
-     * Holds the view state for new/switched tabs
-     */
-    MainWindow::ViewState m_ViewState;
-
     /**
      * Collects signals and sends specific parameters to the connected slots.
      */
-    QSignalMapper *m_headingMapper;
-    QSignalMapper *m_casingChangeMapper;
-    QSignalMapper *m_pluginMapper;
+    QActionGroup *m_casingChangeGroup;
 
     /**
      * The Search Manager dialog
@@ -1002,27 +1048,59 @@ private:
     /**
      * Workaround for Qt 4.8 bug, to track the last known window size when not maximized.
      */
-    QByteArray m_LastWindowSize;
+    bool m_FirstTime;
 
     QTimer m_PreviewTimer;
 
     HTMLResource *m_PreviousHTMLResource;
     QString m_PreviousHTMLText;
-    QList<ViewEditor::ElementIndex> m_PreviousHTMLLocation;
+    QList<ElementIndex> m_PreviousHTMLLocation;
 
     /**
      * dynamically updated plugin menus and actions
      */
-    QMenu *m_menuPlugins;
-    QMenu *m_menuPluginsInput;
-    QMenu *m_menuPluginsOutput;
-    QMenu *m_menuPluginsEdit;
-    QMenu *m_menuPluginsValidation;
-    QAction *m_actionManagePlugins;
+    QMenu *m_menuPlugins = nullptr;
+    QMenu *m_menuPluginsInput = nullptr;
+    QMenu *m_menuPluginsOutput = nullptr;
+    QMenu *m_menuPluginsEdit = nullptr;
+    QMenu *m_menuPluginsValidation = nullptr;
+    QAction *m_actionManagePlugins = nullptr;;
 
     QStringList m_pluginList;
-    bool m_SaveCSS;
 
+    /**
+     * dynamically updated Automation menus and actions
+     */
+    QMenu *m_menuAutomate = nullptr;;
+    QMenu *m_menuAutomateRun = nullptr;
+    QAction *m_actionManageAutomate = nullptr;;
+
+    bool m_SaveTab;
+    bool m_IsClosing;
+
+    QList<QAction*> m_qlactions;
+
+    QList<QAction*> m_clactions;
+
+    QList<QAction*> m_auactions;
+
+
+    /**
+     * Collects signals and sends specific parameters to the connected slots.
+     */
+    QActionGroup *m_headingActionGroup;
+
+    QString m_mathjaxfolder;
+
+    // bool m_FRVisible;
+
+    bool m_UsingAutomate;
+    QStringList m_AutomateLog;
+    QString m_AutomatePluginParameter;
+    QStringList m_automateLists;
+
+    bool m_inShowLastOpenWarnings = false;
+    
     /**
      * Holds all the widgets Qt Designer created for us.
      */
